@@ -982,7 +982,7 @@ void rejustify(short mode)
    labelptr curlabel = NULL;
    short    *tsel;
    short jsave;
-   Boolean changed = False;
+   Boolean preselected = False, changed = False;
    static short transjust[] = {15, 13, 12, 7, 5, 4, 3, 1, 0};
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
@@ -1003,6 +1003,12 @@ void rejustify(short mode)
       setfontmarks(-1, curlabel->justify);
    }
    else {
+      if (areawin->selects == 0) {
+	 if (!checkselect(LABEL))
+	    return;
+      }
+      else preselected = TRUE;
+
       for (tsel = areawin->selectlist; tsel < areawin->selectlist +
 	      areawin->selects; tsel++) {
 	 if (SELECTTYPE(tsel) == LABEL) {
@@ -1018,7 +1024,7 @@ void rejustify(short mode)
 	    }
 	 }
       }
-      if (eventmode != MOVE_MODE && eventmode != COPY_MODE)
+      if (preselected == FALSE && eventmode != MOVE_MODE && eventmode != COPY_MODE)
 	 unselect_all();
       else
 	 draw_all_selected();
@@ -1800,14 +1806,22 @@ void unjoin()
    short *selectobj;
    genericptr *genp, *newg;
    pathptr oldpath;
+   polyptr oldpoly, *newpoly;
+   Boolean preselected;
+   short i, cycle;
+
+   if (areawin->selects == 0) {
+      select_element(PATH | POLYGON);
+      preselected = FALSE;
+   }
+   else preselected = TRUE;
 
-   if (areawin->selects == 0) select_element(PATH);
    if (areawin->selects == 0) {
       Wprintf("No objects selected.");
       return;
    }
 
-   /* for each selected path */
+   /* for each selected path or polygon */
 
    XcSetFunction(GXcopy);
 
@@ -1837,8 +1851,26 @@ void unjoin()
          freepathparts(selectobj, 0);
          reviseselect(areawin->selectlist, areawin->selects, selectobj);
       }
+      else if (SELECTTYPE(selectobj) == POLYGON) {
+	 /* Method to break a polygon, in lieu of the edit-mode	*/
+	 /* polygon break that was removed.			*/
+         oldpoly = SELTOPOLY(selectobj);
+         UDrawPolygon(oldpoly);
+
+	 /* Get the point nearest the cursor, and break at that point */
+         cycle = closepoint(oldpoly, &areawin->save);
+	 if (cycle > 0 && cycle < (oldpoly->number - 1)) {
+            NEW_POLY(newpoly, topobject);
+	    topobject->parts++;
+	    polycopy(*newpoly, oldpoly);
+	    for (i = cycle; i < oldpoly->number; i++)
+	       (*newpoly)->points[i - cycle] = (*newpoly)->points[i];
+	    oldpoly->number = cycle + 1;
+	    (*newpoly)->number = (*newpoly)->number - cycle;
+	 }
+      }
    }
-   clearselects();
+   if (!preselected) clearselects();
    drawarea(NULL, NULL, NULL);
 }
 
@@ -2069,16 +2101,6 @@ void join()
       register_for_undo(XCF_Delete, UNDO_DONE, areawin->topinstance,
                 delobj, NORMAL);
 
-  //  standard_element_delete(NORMAL);
-
-  //   for (sptr = scount; sptr < scount + numpolys; sptr++) {
-  //	 easydraw(*sptr, DOFORALL);
-  //	 freepathparts(sptr, 1);
-  //	 /* revise the list of path elements */
-  //	 for (sptr2 = sptr + 1; sptr2 < scount + numpolys; sptr2++)
-  //	    if (*sptr2 > *sptr) (*sptr2)--;
-  //   }
-
       XcSetForeground((*newpoly)->color);
       UDrawPolygon(*newpoly);
 
@@ -2143,7 +2165,11 @@ void join()
    /* clean up */
 
    incr_changes(topobject);
-   clearselects();
+   /* Do not clear the selection, to be consistent with all the	*/
+   /* other actions that clear only when something has not been	*/
+   /* preselected before the action.  Elements must be selected	*/
+   /* prior to the "join" action, by necessity.			*/
+   /* clearselects(); */
    free(scount);
    free(order);
    free(direc);
