@@ -1995,65 +1995,122 @@ void UDrawBox(XPoint origin, XPoint corner)
 /* closely reach the position "corner".					*/
 /*----------------------------------------------------------------------*/
 
-void UDrawRescaleBox(XPoint *corner)
+float UDrawRescaleBox(XPoint *corner)
 {
    XPoint 	origpoints[5], newpoints[5];
    genericptr	rgen;
-   float	savescale, newscale, lastscale, tempscale;
-   long		mindist, testdist, lastmindist;
+   float	savescale, newscale;
+   long		mindist, testdist, refdist;
    labelptr	rlab;
-   int		i, j; /* , idx; (jdk) */
+   graphicptr   rgraph;
+   objinstptr	rinst;
+   int		i;
 
-   /* To be done---draw a box around each selected part, not	*/
-   /* just the first one.					*/
+   if (areawin->selects == 0) return 0.0;
 
    SetFunction(dpy, areawin->gc, GXxor);
    SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
    
-   for (j = 0; j < areawin->selects; j++) {
-      rgen = SELTOGENERIC(areawin->selectlist + j);
+   /* Use only the 1st selection as a reference to set the scale */
 
-      switch(ELEMENTTYPE(rgen)) {
-         case LABEL:
-	    rlab = (labelptr)rgen;
-	    savescale = rlab->scale;
-	    newscale = lastscale = rlab->scale * 2;
-	    mindist = LONG_MAX;
-	    while (mindist > 4) {
-	       rlab->scale = newscale;
-	       labelbbox(rlab, newpoints, areawin->topinstance);
-	       newpoints[5] = newpoints[0];
-	       lastmindist = mindist;
-	       mindist = LONG_MAX;
-	       for (i = 0; i < 4; i++) {
-	          testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
-	          if (testdist < mindist)
-	             mindist = testdist;
-	       }
-	       if (mindist == lastmindist)
-		  break;
-	       else if (lastmindist == LONG_MAX)
-		  newscale /= 2;
-	       else {
-	          tempscale = (fabs)((newscale * lastmindist) - (lastscale * mindist)) / 
-				(abs)(lastmindist - mindist);
-	          lastscale = newscale;
-	          newscale = tempscale;
-	       }
-	    }
-	    rlab->scale = savescale;
-            break;
-         case GRAPHIC:
-            break;
-         case OBJINST:
-            break;
-         /* Note:  should be able to handle other elements, as well */
-      }
+   rgen = SELTOGENERIC(areawin->selectlist);
 
-      UTransformbyCTM(DCTM, newpoints, origpoints, 4);
-      strokepath(origpoints, 4, 0, 1);
+   switch(ELEMENTTYPE(rgen)) {
+      case LABEL:
+	 rlab = (labelptr)rgen;
+	 labelbbox(rlab, newpoints, areawin->topinstance);
+	 newpoints[4] = newpoints[0];
+	 mindist = LONG_MAX;
+	 for (i = 0; i < 4; i++) {
+	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	    if (testdist < mindist)
+	       mindist = testdist;
+	 }
+	 refdist = wirelength(corner, &(rlab->position));
+	 mindist = (int)sqrt(abs((double)mindist));
+	 savescale = rlab->scale;
+	 if (!test_insideness((int)corner->x, (int)corner->y, newpoints))
+	    mindist = -mindist;
+	 if (refdist == mindist) refdist = 1 - mindist;
+	 newscale = fabs(rlab->scale * (float)refdist / (float)(refdist + mindist));
+	 if (newscale > 10 * rlab->scale) newscale = 10 * rlab->scale;
+	 if (areawin->snapto) {
+	    float snapstep = 2 * (float)xobjs.pagelist[areawin->page]->gridspace
+			/ (float)xobjs.pagelist[areawin->page]->snapspace;
+	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
+	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
+	 }
+	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 rlab->scale = newscale;
+	 labelbbox(rlab, newpoints, areawin->topinstance);
+	 rlab->scale = savescale;
+         break;
+
+      case GRAPHIC:
+	 rgraph = (graphicptr)rgen;
+	 graphicbbox(rgraph, newpoints);
+	 newpoints[4] = newpoints[0];
+	 mindist = LONG_MAX;
+	 for (i = 0; i < 4; i++) {
+	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	    if (testdist < mindist)
+	       mindist = testdist;
+	 }
+	 refdist = wirelength(corner, &(rgraph->position));
+	 mindist = (int)sqrt(abs((double)mindist));
+	 savescale = rgraph->scale;
+	 if (!test_insideness((int)corner->x, (int)corner->y, newpoints))
+	    mindist = -mindist;
+	 if (refdist == mindist) refdist = 1 - mindist;  /* avoid inf result */
+	 newscale = fabs(rgraph->scale * (float)refdist / (float)(refdist + mindist));
+	 if (newscale > 10 * rgraph->scale) newscale = 10 * rgraph->scale;
+	 if (areawin->snapto) {
+	    float snapstep = 2 * (float)xobjs.pagelist[areawin->page]->gridspace
+			/ (float)xobjs.pagelist[areawin->page]->snapspace;
+	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
+	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
+	 }
+	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 rgraph->scale = newscale;
+	 graphicbbox(rgraph, newpoints);
+	 rgraph->scale = savescale;
+         break;
+
+      case OBJINST:
+	 rinst = (objinstptr)rgen;
+	 objinstbbox(rinst, newpoints, FALSE);
+	 newpoints[4] = newpoints[0];
+	 mindist = LONG_MAX;
+	 for (i = 0; i < 4; i++) {
+	    testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	    if (testdist < mindist)
+	       mindist = testdist;
+	 }
+	 refdist = wirelength(corner, &(rinst->position));
+	 mindist = (int)sqrt(abs((double)mindist));
+	 savescale = rinst->scale;
+	 if (!test_insideness((int)corner->x, (int)corner->y, newpoints))
+	    mindist = -mindist;
+	 if (refdist == mindist) refdist = 1 - mindist;  /* avoid inf result */
+	 newscale = fabs(rinst->scale * (float)refdist / (float)(refdist + mindist));
+	 if (newscale > 10 * rinst->scale) newscale = 10 * rinst->scale;
+	 if (areawin->snapto) {
+	    float snapstep = 2 * (float)xobjs.pagelist[areawin->page]->gridspace
+			/ (float)xobjs.pagelist[areawin->page]->snapspace;
+	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
+	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
+	 }
+	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 rinst->scale = newscale;
+	 objinstbbox(rinst, newpoints, FALSE);
+	 rinst->scale = savescale;
+         break;
    }
+
+   UTransformbyCTM(DCTM, newpoints, origpoints, 4);
+   strokepath(origpoints, 4, 0, 1);
+   return newscale;
 }
 
 /*-------------------------------------------------------------------------*/
