@@ -88,7 +88,7 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Text Return", "Text Delete", "Text Right", "Text Left",
    "Text Up", "Text Down", "Text Split", 
    "Text Home", "Text End", "Linebreak", "Parameter",
-   "Parameterize Point", "Break at Point", "Delete Point", "Insert Point",
+   "Parameterize Point", "Change Style", "Delete Point", "Insert Point",
    "Append Point", "Next Point", "Attach", "Next Library", "Library Directory",
    "Library Move", "Library Copy", "Library Edit", "Library Delete",
    "Library Duplicate", "Library Hide", "Library Virtual Copy",
@@ -101,10 +101,9 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Undo", "Redo", "Select Save", "Unselect", "Dashed", "Dotted",
    "Solid", "Prompt", "Dot", "Wire", "Cancel", "Nothing", "Exit",
    "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label", "Graphic",
-   "Rescale", "Select Box", "Connectivity", "Continue Copy",
-   "Continue Element", "Finish Copy", "Finish Element", "Finish",
-   "Cancel Last", "Sim", "SPICE", "PCB", "SPICE Flat", "Color",
-   "Text Delete Param", "Scale", "Change Style",
+   "Select Box", "Connectivity", "Continue Element", "Finish Element",
+   "Continue Copy", "Finish Copy", "Finish", "Cancel Last", "Sim",
+   "SPICE", "PCB", "SPICE Flat", "Rescale", "Text Delete Param",
    NULL			/* sentinel */
 };
 
@@ -135,10 +134,12 @@ int firstbinding(xcWidget window, int function)
    int keywstate = -1;
 
    for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding) {
-      if (ksearch->window == window)
-	 return ksearch->keywstate;
-      else if (ksearch->window == ALL_WINDOWS)
-	 keywstate = ksearch->keywstate;
+      if (ksearch->function == function) {
+         if (ksearch->window == window)
+	    return ksearch->keywstate;
+         else if (ksearch->window == ALL_WINDOWS)
+	    keywstate = ksearch->keywstate;
+      }
    }
    return keywstate;
 }
@@ -473,13 +474,6 @@ void printeditbindings()
 
    _STR2[0] = '\0';
 
-   tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Break));
-   strcat(_STR2, tstr);
-   strcat(_STR2, "=");
-   strcat(_STR2, func_to_string(XCF_Edit_Break));
-   strcat(_STR2, ", ");
-   free(tstr);
-
    tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Delete));
    strcat(_STR2, tstr);
    strcat(_STR2, "=");
@@ -681,7 +675,6 @@ void default_keybindings()
    add_binding(ALL_WINDOWS, XK_backslash, XCF_Special);
    add_binding(ALL_WINDOWS, ALT | XK_c, XCF_Special);
    add_binding(ALL_WINDOWS, XK_p, XCF_Edit_Param);
-   add_binding(ALL_WINDOWS, XK_x, XCF_Edit_Break);
    add_binding(ALL_WINDOWS, XK_d, XCF_Edit_Delete);
    add_binding(ALL_WINDOWS, XK_Delete, XCF_Edit_Delete);
    add_binding(ALL_WINDOWS, XK_i, XCF_Edit_Insert);
