@@ -220,7 +220,7 @@ void changetextscale(float newscale)
 	       undrawtext(settext);
                settext->scale = newscale;
 	       redrawtext(settext);
-	       register_for_undo(XCF_Scale, UNDO_MORE, areawin->topinstance,
+	       register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			(genericptr)settext, (double)oldscale);
 	    }
 	 }
@@ -488,7 +488,7 @@ void setosize(xcWidget w, objinstptr dataptr)
          nsobj->scale = tmpres;
 
          if (oldsize != tmpres)
-            register_for_undo(XCF_Scale, UNDO_MORE, areawin->topinstance,
+            register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(osel), (double)oldsize);
       }
    }
@@ -543,7 +543,7 @@ void setwwidth(xcWidget w, void *dataptr)
          }
 
 	 if (oldwidth != tmpres)
-            register_for_undo(XCF_Scale, UNDO_MORE, areawin->topinstance,
+            register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(osel), (double)oldwidth);
       }
       unselect_all();
@@ -923,10 +923,19 @@ polyptr checkforbbox(objectptr localdata)
 
 int setelementstyle(xcWidget w, u_short value, u_short mask)
 {
-   Boolean selected = False;
+   Boolean preselected, selected = False;
    short *sstyle;
    u_short newstyle, oldstyle;
 
+   if (areawin->selects == 0) {
+      preselected = FALSE;
+      if (value & BBOX)
+	 checkselect(POLYGON);
+      else
+	 checkselect(ARC | SPLINE | POLYGON | PATH);
+   }
+   else preselected = TRUE;
+
    if (areawin->selects > 0) {
       if (value & BBOX) {
 	 polyptr ckp;
@@ -1026,8 +1035,10 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
    }
 #ifndef TCL_WRAPPER
    setallstylemarks(newstyle);
-   unselect_all();
 #endif
+   if (!preselected)
+      unselect_all();
+
    return (int)newstyle;
 }
 
@@ -1383,6 +1394,7 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
    short *fselect;
    labelptr settext;
    short labelcount = 0;
+   Boolean preselected;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
 		eventmode == EFONTCAT_MODE) return;
@@ -1393,6 +1405,11 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
       charreport(settext);
    }
    else {
+      if (areawin->selects == 0) {
+	 checkselect(LABEL);
+	 preselected = FALSE;
+      }
+      else preselected = TRUE;
       areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
@@ -1403,7 +1420,7 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
          }
       }
       if (labelcount == 0) setfontval(w, value, NULL);
-      else unselect_all();
+      else if (!preselected) unselect_all();
    }
 }
 
@@ -1479,6 +1496,7 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
    short *fselect;
    labelptr settext;
    short labelcount = 0;
+   Boolean preselected;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
 		eventmode == EFONTCAT_MODE) return;
@@ -1489,6 +1507,11 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
       charreport(settext);
    }
    else {
+      if (areawin->selects == 0) {
+	 checkselect(LABEL);
+	 preselected = FALSE;
+      }
+      else preselected = TRUE;
       areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
@@ -1499,7 +1522,7 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setfontstyle(w, value, NULL);
-      else unselect_all();
+      else if (!preselected) unselect_all();
    }
 }
 
@@ -1574,6 +1597,7 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
    short *fselect;
    labelptr settext;
    short labelcount = 0;
+   Boolean preselected;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
 		eventmode == EFONTCAT_MODE) return;
@@ -1584,6 +1608,11 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
       charreport(settext);
    }
    else {
+      if (areawin->selects == 0) {
+  	 checkselect(LABEL);
+	 preselected = FALSE; 
+      }
+      else preselected = TRUE;
       areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
@@ -1594,6 +1623,7 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setfontencoding(w, value, NULL);
+      else if (!preselected) unselect_all();
    }
 }
 
