@@ -397,8 +397,6 @@ void setparammarks(genericptr thiselem)
    for (i = 0; i < NUM_PARAM_TYPES; i++)
       ptest[i] = FALSE;
 
-   XcFakeTagCall(xcinterp, 2, "parameter", "reset");
-
    /* For each parameter declared, set the corresponding Tcl variable */
    if (thiselem != NULL) {
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
@@ -575,12 +573,12 @@ int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
 	       break;
 	    case XC_FLOAT:
 	       fval = dps->parameter.fvalue;
-	       ival = (int)(fval + 0.5);
+	       ival = (int)(fval + ((fval < 0) ? -0.5 : 0.5));
 	       break;
 	    case XC_STRING:
 	       promoted = textprint(dps->parameter.string, pinst);
 	       if (sscanf(promoted, "%g", &fval) == 1)
-		  ival = (int)(fval + 0.5);
+	          ival = (int)(fval + ((fval < 0) ? -0.5 : 0.5));
 	       else
 		  ival = 0;
 	       free(promoted);
@@ -588,7 +586,7 @@ int epsubstitute(genericptr thiselem, objectptr thisobj, objinstptr pinst,
 	    case XC_EXPR:
 	       if ((promoted = evaluate_expr(thisobj, dps, pinst)) == NULL) continue;
 	       if (sscanf(promoted, "%g", &fval) == 1)
-		  ival = (int)(fval + 0.5);
+	          ival = (int)(fval + ((fval < 0) ? -0.5 : 0.5));
 	       free(promoted);
 	       break;
 	 }
