@@ -434,7 +434,7 @@ extern void UDrawTextLine(labelptr, short);
 extern void UDrawTLine(labelptr);
 extern void UDrawXLine(XPoint, XPoint);
 extern void UDrawBox(XPoint, XPoint);
-extern void UDrawRescaleBox(XPoint *);
+extern float UDrawRescaleBox(XPoint *);
 extern void UDrawBBox(void);
 extern void strokepath(XPoint *, short, short, float);
 extern void makesplinepath(splineptr, XPoint *);
