@@ -873,8 +873,7 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 {
    selection *rselect, *rcheck, *lastselect;
    genericptr rgen;
-   short i; /* , *selectobj, numselect, origselect; (jdk) */
-   short rclass;
+   short i;
    objectptr selobj;
    objinstptr selinst;
    XPoint savesave, tmppt;
@@ -923,9 +922,7 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 			areawin->save.x, areawin->save.y); */
          UPushCTM();
          UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
-	 rclass = class;
-	 if ((class & (~OBJINST)) == 0) class = ALL_TYPES;
-         rcheck = recurselect(class, recmode, &selnew);
+         rcheck = recurselect(ALL_TYPES, recmode, &selnew);
          UPopCTM();
          areawin->save.x = savesave.x;
          areawin->save.y = savesave.y;
@@ -1013,7 +1010,6 @@ void trackselarea()
 void trackrescale()
 {
    XPoint newpos;
-   /* u_int  nullui; (jdk) */
 
    newpos = UGetCursorPos();
    if (newpos.x == areawin->save.x && newpos.y == areawin->save.y) return;
