@@ -52,12 +52,31 @@ typedef struct {
    short yaxis;
 } arcinfo;
 
+/* Smaller data structure used when saving paths in the undo stacks.    */
+/* We don't need the rendering data from points[].                      */
+
 typedef struct {
+   u_short	type;
+   int		color;
+   eparamptr	passed;
+   short	cycle;
+   u_short	style;
+   float	width;
+   XPoint	ctrl[4];
+} splineinfo;
+
+typedef struct {
+   int number;
+   pointlist points;
+} pathinfo;
+
+typedef struct editelem {
    genericptr element;		/* element being edited */
    union {
       stringpart *string;	/* original contents of string, for label */
       pointlist points;		/* original polygon */
       arcinfo *arcspecs;	/* original arc values */
+      pathinfo *pathspecs;	/* original parts of a path */
    } save;
 } editelement;
 
@@ -183,8 +202,12 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
 /*   Free memory allocated to an undo record edit element structure.	*/
 /*----------------------------------------------------------------------*/
 
-void free_editelement(editelement *erec)
+void free_editelement(Undoptr thisrecord)
 {
+   editelement *erec;
+   pathinfo *ppi;
+
+   erec = (editelement *)thisrecord->undodata;
    switch (erec->element->type) {
       case LABEL:
 	 freelabel(erec->save.string);
@@ -195,6 +218,12 @@ void free_editelement(editelement *erec)
       case ARC:
 	 free(erec->save.arcspecs);
 	 break;
+      case PATH:
+	 for (ppi = erec->save.pathspecs; ppi < erec->save.pathspecs +
+		thisrecord->idata; ppi++)
+	    free(ppi->points);
+	 free(erec->save.pathspecs);
+	 break;
    }
    free(erec);
 }
@@ -349,7 +378,7 @@ void undelete_one_element(objinstptr thisinst, genericptr thiselem)
 void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 {
    va_list args;
-   int drawmode, nval, oval, *idata, snum, deltax, deltay, dir; /*  usize, (jdk) */
+   int drawmode, nval, oval, *idata, snum, deltax, deltay, dir, i;
    short *slist;
    objectptr delobj;
    objinstptr newinst;
@@ -357,7 +386,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    uselection *srec;
    editelement *erec;
    scaleinfo *escale;
-   genericptr egen; /* , newgen; (jdk) */
+   genericptr egen, *ggen, *cgen;
    XPoint *fpoint;
    double scale;
 
@@ -490,6 +519,27 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	       erec->save.arcspecs->radius = ((arcptr)egen)->radius;
 	       erec->save.arcspecs->yaxis = ((arcptr)egen)->yaxis;
 	       break;
+	    case PATH:
+	       newrecord->idata = ((pathptr)egen)->parts;  /* not needed? */
+	       erec->save.pathspecs = (pathinfo *)malloc(newrecord->idata *
+			sizeof(pathinfo));
+	       for (i = 0; i < newrecord->idata; i++) {
+		  pathinfo *ppi = erec->save.pathspecs + i;
+		  genericptr *pgen = ((pathptr)egen)->plist + i;
+		  switch (ELEMENTTYPE(*pgen)) {
+		     case POLYGON:
+			ppi->number = (TOPOLY(pgen))->number;
+			ppi->points = copypoints((TOPOLY(pgen))->points,
+				ppi->number);
+			break;
+		     case SPLINE:
+			ppi->number = 4;
+			ppi->points = copypoints((pointlist)
+				(TOSPLINE(pgen))->ctrl, 4);
+			break;
+		  }
+	       }
+	       break;
 	 }
 	 newrecord->undodata = (char *)erec;
 	 break;
@@ -505,7 +555,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	 newrecord->idata = oval;
 	 break;
 
-      case XCF_Scale:
+      case XCF_Rescale:
 	 /* 2 args:							*/
 	 /*	egen = element that was changed (with new value)	*/
 	 /*	ofloat = old float value				*/
@@ -573,7 +623,7 @@ short undo_one_action()
    scaleinfo *escale;
    short *slist;
    XPoint *delta, position;
-   int i, snum; /* , npage, opage; (jdk) */
+   int i, j, snum;
    int savemode;
    float fnum;
    genericptr egen;
@@ -719,6 +769,21 @@ short undo_one_action()
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areawin->area, NULL, NULL);
+	    }
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -742,19 +807,36 @@ short undo_one_action()
 	       calcspline(espline);
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
-	    case ARC: {
-	       arcinfo tmpinfo;
-	       arcptr earc = (arcptr)(erec->element);
-	       tmpinfo.angle1 = earc->angle1; 
-	       tmpinfo.angle2 = earc->angle2; 
-	       tmpinfo.radius = earc->radius; 
-	       tmpinfo.yaxis = earc->yaxis; 
-	       earc->angle1 = erec->save.arcspecs->angle1;
-	       earc->angle2 = erec->save.arcspecs->angle2;
-	       earc->radius = erec->save.arcspecs->radius;
-	       earc->yaxis = erec->save.arcspecs->yaxis;
-	       *(erec->save.arcspecs) = tmpinfo;
-	       calcarc(earc);
+	    case PATH: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly;
+	       splineptr espline;
+	       pathinfo *tmpinfo = erec->save.pathspecs;
+	       pathptr epath = (pathptr)(erec->element);
+	       for (i = 0; i < epath->parts; i++) {
+		  genericptr ggen = *(epath->plist + i);
+		  switch (ELEMENTTYPE(ggen)) {
+		     case POLYGON:
+			epoly = (polyptr)ggen;
+		        tmppts = epoly->points;
+			tmpnum = epoly->number;
+			epoly->points = (erec->save.pathspecs + i)->points;
+			epoly->number = (erec->save.pathspecs + i)->number;
+			(erec->save.pathspecs + i)->points = tmppts;
+			(erec->save.pathspecs + i)->number = tmpnum;
+			break;
+		     case SPLINE:
+			espline = (splineptr)ggen;
+			tmppts = copypoints((pointlist)espline->ctrl, 4);
+			for (j = 0; j < 4; j++)
+			   espline->ctrl[j] = *((erec->save.pathspecs + i)->points + j);
+			free((erec->save.pathspecs + i)->points);
+		        (erec->save.pathspecs + i)->points = tmppts;
+			calcspline(espline);
+			break;
+		  }
+	       }
 	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
@@ -811,7 +893,7 @@ short undo_one_action()
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
-      case XCF_Scale:
+      case XCF_Rescale:
 	 escale = (scaleinfo *)thisrecord->undodata;
 	 egen = escale->element;
 	 fnum = escale->scale;
@@ -847,6 +929,11 @@ short undo_one_action()
 	       thisgraphic->scale = fnum;
 	       thisgraphic->valid = FALSE;
 	       break;
+	    case LABEL:
+	       thislabel = (labelptr)egen;
+	       escale->scale = thislabel->scale;
+	       thislabel->scale = fnum;
+	       break;
 	 }
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
@@ -868,7 +955,9 @@ short undo_one_action()
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
+	 select_connected_pins();
 	 placeselects(-(delta->x), -(delta->y), NULL);
+	 reset_connected_pins();
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -924,7 +1013,7 @@ short redo_one_action()
    uselection *srec;
    editelement *erec;
    scaleinfo *escale;
-   int i, snum; /*  tpage, (jdk) */
+   int i, j, snum;
    int savemode;
    float fnum;
    labelptr thislabel;
@@ -1056,6 +1145,21 @@ short redo_one_action()
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areawin->area, NULL, NULL);
+	    }
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -1079,19 +1183,36 @@ short redo_one_action()
 	       calcspline(espline);
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
-	    case ARC: {
-	       arcinfo tmpinfo;
-	       arcptr earc = (arcptr)(erec->element);
-	       tmpinfo.angle1 = earc->angle1; 
-	       tmpinfo.angle2 = earc->angle2; 
-	       tmpinfo.radius = earc->radius; 
-	       tmpinfo.yaxis = earc->yaxis; 
-	       earc->angle1 = erec->save.arcspecs->angle1;
-	       earc->angle2 = erec->save.arcspecs->angle2;
-	       earc->radius = erec->save.arcspecs->radius;
-	       earc->yaxis = erec->save.arcspecs->yaxis;
-	       *(erec->save.arcspecs) = tmpinfo;
-	       calcarc(earc);
+	    case PATH: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly;
+	       splineptr espline;
+	       pathinfo *tmpinfo = erec->save.pathspecs;
+	       pathptr epath = (pathptr)(erec->element);
+	       for (i = 0; i < epath->parts; i++) {
+		  genericptr ggen = *(epath->plist + i);
+		  switch (ELEMENTTYPE(ggen)) {
+		     case POLYGON:
+			epoly = (polyptr)ggen;
+		        tmppts = epoly->points;
+			tmpnum = epoly->number;
+			epoly->points = (erec->save.pathspecs + i)->points;
+			epoly->number = (erec->save.pathspecs + i)->number;
+			(erec->save.pathspecs + i)->points = tmppts;
+			(erec->save.pathspecs + i)->number = tmpnum;
+			break;
+		     case SPLINE:
+			espline = (splineptr)ggen;
+			tmppts = copypoints((pointlist)espline->ctrl, 4);
+			for (j = 0; j < 4; j++)
+			   espline->ctrl[j] = *((erec->save.pathspecs + i)->points + j);
+			free((erec->save.pathspecs + i)->points);
+		        (erec->save.pathspecs + i)->points = tmppts;
+			calcspline(espline);
+			break;
+		  }
+	       }
 	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
@@ -1136,7 +1257,7 @@ short redo_one_action()
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
-      case XCF_Scale:
+      case XCF_Rescale:
 	 escale = (scaleinfo *)thisrecord->undodata;
 	 egen = escale->element;
 	 fnum = escale->scale;
@@ -1172,6 +1293,11 @@ short redo_one_action()
 	       thisgraphic->scale = fnum;
 	       thisgraphic->valid = FALSE;
 	       break;
+	    case LABEL:
+	       thislabel = (labelptr)egen;
+	       escale->scale = thislabel->scale;
+	       thislabel->scale = fnum;
+	       break;
 	 }
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
@@ -1183,7 +1309,9 @@ short redo_one_action()
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
+	 select_connected_pins();
 	 placeselects(delta->x, delta->y, NULL);
+	 reset_connected_pins();
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1309,7 +1437,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
       case XCF_Edit:
 	 erec = (editelement *)thisrecord->undodata;
-	 free_editelement(erec);
+	 free_editelement(thisrecord);
 	 break;
 
       case XCF_Copy:
